@@ -138,6 +138,11 @@ class TripDddWriter(object):
         if not self.ddd_filename.endswith(".ddd"):
             self.ddd_filename += ".ddd"
         self.outputdir = os.path.abspath(os.path.dirname(self.ddd_filename))
+        self.threshold = 3e-3
+        env_var_name = 'DDD_TAIL_THRESHOLD'
+        if env_var_name in os.environ:
+            self.threshold = float(os.environ[env_var_name])
+            logger.info("Setting tail threshold based on {:s} to {:f}".format(env_var_name, self.threshold))
 
     def write(self, detector):
         from pymchelper.shieldhit.detector.detector_type import SHDetType
@@ -147,16 +152,40 @@ class TripDddWriter(object):
                 detector.dettyp, SHDetType.ddd))
             return 1
 
+        # guess projectile and energy
+        if self.projectile == '0':
+            try:
+                element_names = ['n', 'H', 'He', 'Li', 'Be', 'B', 'C', 'N', 'O', 'F', 'Ne', 'Na', 'Mg', 'Al', 'Si',
+                                 'P', 'S', 'Cl', 'Ar', 'K', 'Ca', 'Sc', 'Ti', 'V', 'Cr', 'Mn', 'Fe', 'Co', 'Ni', 'Cu',
+                                 'Zn', 'Ga', 'Ge', 'As', 'Se', 'Br', 'Kr', 'Rb', 'Sr', 'Y', 'Zr', 'Nb', 'Mo', 'Tc',
+                                 'Ru', 'Rh', 'Pd', 'Ag', 'Cd', 'In', 'Sn', 'Sb', 'Te', 'I', 'Xe', 'Cs', 'Ba', 'La',
+                                 'Ce', 'Pr', 'Nd', 'Pm', 'Sm', 'Eu', 'Gd', 'Tb', 'Dy', 'Ho', 'Er', 'Tm', 'Yb', 'Lu',
+                                 'Hf', 'Ta', 'W', 'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg', 'Tl', 'Pb', 'Bi', 'Po', 'At',
+                                 'Rn', 'Fr', 'Ra', 'Ac', 'Th', 'Pa', 'U', 'Np', 'Pu', 'Am', 'Cm', 'Bk', 'Cf', 'Es',
+                                 'Fm', 'Md', 'No', 'Lr', 'Rf', 'Db', 'Sg', 'Bh', 'Hs', 'Mt', 'Ds', 'Rg', 'Cn', 'Nh',
+                                 'Fl', 'Mc', 'Lv', 'Ts', 'Og']
+                self.projectile = "{:d}{:s}".format(
+                    int(detector.projectile_a),
+                    element_names[int(detector.projectile_z)],
+                )
+            except AttributeError:
+                logger.error('Projectile energy not available in raw_data, setting to 0')
+
+        if self.energy_MeV == 0:
+            try:
+                self.energy_MeV = detector.projectile_energy
+            except AttributeError:
+                logger.error('Projectile energy not available in raw_data, setting to 0')
+
         # extract data from detector data
         self._extract_data(detector)
 
         # in order to avoid fitting data to noisy region far behind Bragg peak tail,
         # find the range of z coordinate which containes (1-threshold) of the deposited energy
-        threshold = 3e-3
         cum_dose = self._cumulative_dose()
         cum_dose_left = self._cumulative_dose_left(cum_dose)
 
-        thr_ind = cum_dose_left.size - np.searchsorted(cum_dose_left[::-1], threshold) - 1
+        thr_ind = cum_dose_left.size - np.searchsorted(cum_dose_left[::-1], self.threshold) - 1
         z_fitting_cm_1d = self.z_data_cm_1d[:thr_ind]
         dose_fitting_MeV_g_1d = self.dose_data_MeV_g_1d[:thr_ind]
 
@@ -169,7 +198,7 @@ class TripDddWriter(object):
                 cum_dose_left=cum_dose_left,
                 z_fitting_cm_1d=z_fitting_cm_1d,
                 dose_fitting_MeV_g_1d=dose_fitting_MeV_g_1d,
-                threshold=threshold,
+                threshold=self.threshold,
                 zmax_cm=z_fitting_cm_1d[-1])
 
             self._plot_2d_map(z_fitting_cm_2d, r_fitting_cm_2d, dose_fitting_MeV_g_2d, z_fitting_cm_1d)
@@ -196,8 +225,6 @@ class TripDddWriter(object):
                 # perform the fit
                 params, params_error = self._lateral_fit(r_fitting_cm,
                                                          dose_fitting_1d_positive_MeV_g,
-                                                         z_cm,
-                                                         self.energy_MeV,
                                                          self.ngauss)
 
                 fwhm1_cm, factor, fwhm2_cm, dz0_MeV_cm_g = params
@@ -251,8 +278,8 @@ class TripDddWriter(object):
             fileversion='19980520',
             filedate=time.strftime('%c'),  # Locale's appropriate date and time representation
             projectile=self.projectile,
-            material='H20',
-            composition='H20',
+            material='H2O',
+            composition='H2O',
             density=1,
             creator=_creator_info,
             energy=self.energy_MeV)
@@ -545,7 +572,7 @@ class TripDddWriter(object):
         if self.ngauss == 2:
             line_objs += lns2
             line_objs += lns3
-        labels = [l.get_label() for l in line_objs]
+        labels = [line_obj.get_label() for line_obj in line_objs]
         ax1.legend(line_objs, labels, loc=0)
 
         out_filename = prefix + 'fwhm.png'
@@ -570,7 +597,7 @@ class TripDddWriter(object):
         # we calculate D(z) which will correspond to depth-dose profile measured with ion. chamber of radius rmax
         # it is basically energy E(z) deposited in slice of radius rmax and thickness dz divided by slice mass
         # D(z) = E(z) / m(z) = E(z) / (pi rmax^2 dz rho)
-        # energy E(z) is the sum of energy in all cylyndrical shell in a slice and can be calculated as integral
+        # energy E(z) is the sum of energy in all cylindrical shell in a slice and can be calculated as integral
         # thin shell has surface at radius r has surface: 2 pi r dr, thus
         # E(z) = \int_0^rmax D(r,z) rho dz 2 pi r dr
         # finally:
@@ -655,9 +682,9 @@ class TripDddWriter(object):
     @classmethod
     def gauss2_MeV_g(cls, x_cm, amp_MeV_cm_g, sigma1_cm, weight, sigma2_add_cm):
         return amp_MeV_cm_g / (2.0 * np.pi) * (
-            (weight / sigma1_cm) * np.exp(-x_cm ** 2 / (2.0 * sigma1_cm ** 2)) +
-            ((1.0 - weight) / (sigma1_cm + sigma2_add_cm)) *
-            np.exp(-x_cm ** 2 / (2.0 * (sigma1_cm + sigma2_add_cm) ** 2))
+            (weight / sigma1_cm) * np.exp(-x_cm ** 2 / (2.0 * sigma1_cm ** 2))
+            + ((1.0 - weight) / (sigma1_cm + sigma2_add_cm))
+            * np.exp(-x_cm ** 2 / (2.0 * (sigma1_cm + sigma2_add_cm) ** 2))
         )
 
     @classmethod
@@ -674,40 +701,45 @@ class TripDddWriter(object):
         return cls.gauss2_MeV_g(x_cm, amp_MeV_cm_g, sigma1_cm, weight, sigma2_add_cm) * x_cm
 
     @classmethod
-    def _lateral_fit(cls, r_cm, dose_MeV_g, z_cm, energy_MeV, ngauss=2):
+    def _lateral_fit(cls, r_cm, dose_MeV_g, ngauss=2):
         variance = np.average(r_cm ** 2, weights=dose_MeV_g)
 
-        starting_amp_MeV_g = dose_MeV_g.max()
-        starting_sigma_cm = np.sqrt(variance)
+        start_amp_MeV_g = dose_MeV_g.max()
+        start_sigma_cm = np.sqrt(variance)
 
         min_amp_MeV_g = 1e-10 * dose_MeV_g.max()
-        min_sigma_cm = 1e-2 * starting_sigma_cm
+        min_sigma_cm = 1e-2 * start_sigma_cm
 
         max_amp_MeV_g = 2.0 * dose_MeV_g.max()
-        max_sigma_cm = 1e4 * starting_sigma_cm
+        max_sigma_cm = 1e4 * start_sigma_cm
 
         from scipy.optimize import curve_fit
 
         if ngauss == 1:
-            popt, pcov = curve_fit(f=cls.gauss_r_MeV_cm_g,
-                                   xdata=r_cm,
-                                   ydata=dose_MeV_g * r_cm,
-                                   p0=[starting_amp_MeV_g, starting_sigma_cm],
-                                   bounds=([[min_amp_MeV_g, min_sigma_cm], [max_amp_MeV_g, max_sigma_cm]]),
-                                   sigma=None)
-            # TODO return also parameter errors
-            perr = np.sqrt(np.diag(pcov))
+            try:
+                popt, pcov = curve_fit(f=cls.gauss_r_MeV_cm_g,
+                                       xdata=r_cm,
+                                       ydata=dose_MeV_g * r_cm,
+                                       p0=[start_amp_MeV_g, start_sigma_cm],
+                                       bounds=([[min_amp_MeV_g, min_sigma_cm], [max_amp_MeV_g, max_sigma_cm]]),
+                                       sigma=None)
+                # TODO return also parameter errors
+                perr = np.sqrt(np.diag(pcov))
 
-            dz0_MeV_cm_g, sigma_cm = popt
-            dz0_MeV_cm_g_error, sigma_cm_error = perr
+                dz0_MeV_cm_g, sigma_cm = popt
+                dz0_MeV_cm_g_error, sigma_cm_error = perr
+            except RuntimeError as e:
+                logger.warning(e)
+                dz0_MeV_cm_g, sigma_cm = np.nan, np.nan
+                dz0_MeV_cm_g_error, sigma_cm_error = np.nan, np.nan
             factor = 0.0
             factor_error = 0.0
             fwhm2_cm = 0.0
             fwhm2_cm_error = 0.0
 
         elif ngauss == 2:
-            starting_weigth = 0.99
-            starting_sigma2_add_cm = 0.1
+            start_weigth = 0.99
+            start_sigma2_add_cm = 0.1
 
             min_weigth = 0.55
             min_sigma2_add_cm = 1e-1
@@ -715,20 +747,24 @@ class TripDddWriter(object):
             max_weigth = 1.0 - 1e-12
             max_sigma2_add_cm = 20.0
 
-            popt, pcov = curve_fit(f=cls.gauss2_r_MeV_cm_g,
-                                   xdata=r_cm,
-                                   ydata=dose_MeV_g * r_cm,
-                                   p0=[starting_amp_MeV_g, starting_sigma_cm, starting_weigth, starting_sigma2_add_cm],
-                                   bounds=([min_amp_MeV_g, min_sigma_cm, min_weigth, min_sigma2_add_cm],
-                                           [max_amp_MeV_g, max_sigma_cm, max_weigth, max_sigma2_add_cm]),
-                                   sigma=None)
+            try:
+                popt, pcov = curve_fit(f=cls.gauss2_r_MeV_cm_g,
+                                       xdata=r_cm,
+                                       ydata=dose_MeV_g * r_cm,
+                                       p0=[start_amp_MeV_g, start_sigma_cm, start_weigth, start_sigma2_add_cm],
+                                       bounds=([min_amp_MeV_g, min_sigma_cm, min_weigth, min_sigma2_add_cm],
+                                               [max_amp_MeV_g, max_sigma_cm, max_weigth, max_sigma2_add_cm]),
+                                       sigma=None)
+                perr = np.sqrt(np.diag(pcov))
+                dz0_MeV_cm_g_error, sigma_cm_error, factor_error, sigma2_add_cm_error = perr
+                dz0_MeV_cm_g, sigma_cm, factor, sigma2_add_cm = popt
+            except RuntimeError as e:
+                logger.warning(e)
+                dz0_MeV_cm_g_error, sigma_cm_error, factor_error, sigma2_add_cm_error = np.nan, np.nan, np.nan, np.nan
+                dz0_MeV_cm_g, sigma_cm, factor, sigma2_add_cm = np.nan, np.nan, np.nan, np.nan
             # TODO return also parameter errors
-            perr = np.sqrt(np.diag(pcov))
-            dz0_MeV_cm_g_error, sigma_cm_error, factor_error, sigma2_add_cm_error = perr
-
-            dz0_MeV_cm_g, sigma_cm, factor, sigma2_add_cm = popt
             sigma2_cm = sigma_cm + sigma2_add_cm
-            sigma2_cm_error = np.sqrt(sigma_cm_error**2 + sigma2_add_cm_error**2)
+            sigma2_cm_error = (sigma_cm_error**2 + sigma2_add_cm_error**2)**0.5
             fwhm2_cm = sigma2_cm * cls._sigma_to_fwhm
             fwhm2_cm_error = sigma2_cm_error * cls._sigma_to_fwhm
 
